@@ -1,102 +1,103 @@
-# -*- coding: utf-8 -*-
 """Environment for the xonsh shell."""
-import os
-import re
-import sys
-import pprint
-import textwrap
-import locale
-import glob
-import threading
-import warnings
-import contextlib
 import collections.abc as cabc
-import subprocess
+import contextlib
+import inspect
+import locale
+import os
 import platform
+import pprint
+import re
+import subprocess
+import sys
+import textwrap
+import threading
 import typing as tp
+import warnings
 from collections import ChainMap
 
+import xonsh.prompt.base as prompt
 from xonsh import __version__ as XONSH_VERSION
-from xonsh.lazyasd import lazyobject, LazyBool
-from xonsh.codecache import run_script_with_cache
-from xonsh.dirstack import _get_cwd
-from xonsh.events import events
-from xonsh.platform import (
-    BASH_COMPLETIONS_DEFAULT,
-    DEFAULT_ENCODING,
-    PATH_DEFAULT,
-    ON_WINDOWS,
-    ON_LINUX,
-    ON_CYGWIN,
-    os_environ,
-)
-from xonsh.built_ins import XSH
-from xonsh.tools import (
-    always_true,
-    always_false,
-    detype,
-    ensure_string,
-    is_path,
-    str_to_path,
-    path_to_str,
-    is_env_path,
-    str_to_env_path,
-    env_path_to_str,
-    is_bool,
-    to_bool,
-    bool_to_str,
-    is_bool_or_none,
-    to_bool_or_none,
-    bool_or_none_to_str,
-    is_history_tuple,
-    to_history_tuple,
-    history_tuple_to_str,
-    is_float,
-    is_string,
-    is_string_or_callable,
-    is_completions_display_value,
-    to_completions_display_value,
-    is_completion_mode,
-    to_completion_mode,
-    is_string_set,
-    csv_to_set,
-    set_to_csv,
-    is_int,
-    to_bool_or_int,
-    bool_or_int_to_str,
-    DefaultNotGiven,
-    print_exception,
-    intensify_colors_on_win_setter,
-    is_dynamic_cwd_width,
-    to_dynamic_cwd_tuple,
-    dynamic_cwd_tuple_to_str,
-    is_logfile_opt,
-    to_logfile_opt,
-    logfile_opt_to_str,
-    executables_in,
-    is_nonstring_seq_of_strings,
-    pathsep_to_upper_seq,
-    seq_to_upper_pathsep,
-    print_color,
-    is_history_backend,
-    to_itself,
-    swap_values,
-    ptk2_color_depth_setter,
-    is_str_str_dict,
-    to_str_str_dict,
-    dict_to_str,
-    to_int_or_none,
-    DefaultNotGivenType,
-    to_repr_pretty_,
-)
 from xonsh.ansi_colors import (
     ansi_color_escape_code_to_name,
     ansi_color_name_to_escape_code,
     ansi_reverse_style,
     ansi_style_by_name,
 )
-import xonsh.prompt.base as prompt
-from xonsh.prompt.gitstatus import _DEFS as GITSTATUS_FIELD_DEFS
+from xonsh.built_ins import XSH
+from xonsh.codecache import run_script_with_cache
+from xonsh.dirstack import _get_cwd
+from xonsh.events import events
+from xonsh.lazyasd import LazyBool, lazyobject
+from xonsh.platform import (
+    BASH_COMPLETIONS_DEFAULT,
+    DEFAULT_ENCODING,
+    ON_CYGWIN,
+    ON_LINUX,
+    ON_WINDOWS,
+    PATH_DEFAULT,
+    os_environ,
+)
+from xonsh.tools import (
+    DefaultNotGiven,
+    DefaultNotGivenType,
+    adjust_shlvl,
+    always_false,
+    always_true,
+    bool_or_int_to_str,
+    bool_or_none_to_str,
+    bool_to_str,
+    csv_to_set,
+    detype,
+    dict_to_str,
+    dynamic_cwd_tuple_to_str,
+    ensure_string,
+    env_path_to_str,
+    executables_in,
+    history_tuple_to_str,
+    intensify_colors_on_win_setter,
+    is_bool,
+    is_bool_or_none,
+    is_completion_mode,
+    is_completions_display_value,
+    is_dynamic_cwd_width,
+    is_env_path,
+    is_float,
+    is_history_backend,
+    is_history_tuple,
+    is_int,
+    is_logfile_opt,
+    is_nonstring_seq_of_strings,
+    is_path,
+    is_str_str_dict,
+    is_string,
+    is_string_or_callable,
+    is_string_set,
+    is_valid_shlvl,
+    logfile_opt_to_str,
+    path_to_str,
+    pathsep_to_upper_seq,
+    print_color,
+    print_exception,
+    ptk2_color_depth_setter,
+    seq_to_upper_pathsep,
+    set_to_csv,
+    str_to_env_path,
+    str_to_path,
+    swap_values,
+    to_bool,
+    to_bool_or_int,
+    to_bool_or_none,
+    to_completion_mode,
+    to_completions_display_value,
+    to_dynamic_cwd_tuple,
+    to_history_tuple,
+    to_int_or_none,
+    to_itself,
+    to_logfile_opt,
+    to_repr_pretty_,
+    to_shlvl,
+    to_str_str_dict,
+)
 
 events.doc(
     "on_envvar_new",
@@ -180,7 +181,7 @@ def locale_convert(key):
             locale.setlocale(LOCALE_CATS[key], val)
             val = locale.setlocale(LOCALE_CATS[key])
         except (locale.Error, KeyError):
-            msg = "Failed to set locale {0!r} to {1!r}".format(key, val)
+            msg = f"Failed to set locale {key!r} to {val!r}"
             warnings.warn(msg, RuntimeWarning)
         return val
 
@@ -390,7 +391,7 @@ class LsColors(cabc.MutableMapping):
         return str(self._d)
 
     def __repr__(self):
-        return "{0}.{1}(...)".format(self.__class__.__module__, self.__class__.__name__)
+        return f"{self.__class__.__module__}.{self.__class__.__name__}(...)"
 
     _repr_pretty_ = to_repr_pretty_
 
@@ -481,10 +482,15 @@ class LsColors(cabc.MutableMapping):
         # run dircolors
         try:
             out = subprocess.check_output(
-                cmd, env=denv, universal_newlines=True, stderr=subprocess.DEVNULL
+                cmd, env=denv, text=True, stderr=subprocess.DEVNULL
             )
         except (subprocess.CalledProcessError, FileNotFoundError):
             return cls(cls.default_settings)
+        except OSError:
+            # necessary to catch OSError: [WinError 740] The requested operation requires elevation
+            if ON_WINDOWS:
+                return cls(cls.default_settings)
+            raise
         if not out:
             return cls(cls.default_settings)
         s = out.splitlines()[0]
@@ -565,12 +571,41 @@ def xonsh_data_dir(env):
 
 @default_value
 def xonsh_config_dir(env):
-    """Ensures and returns the $XONSH_CONFIG_DIR"""
+    """``$XDG_CONFIG_HOME/xonsh``"""
     xcd = os.path.expanduser(os.path.join(env.get("XDG_CONFIG_HOME"), "xonsh"))
     os.makedirs(xcd, exist_ok=True)
     return xcd
 
 
+@default_value
+def xdg_data_dirs(env):
+    r"""
+    On Windows: ``[%ProgramData%]`` (normally C:\ProgramData)
+        - More Info: https://docs.microsoft.com/en-us/windows-hardware/customize/desktop/unattend/microsoft-windows-shell-setup-folderlocations-programdata
+
+    On Linux and Unix based systemd it is the same as in open-desktop standard: ``['/usr/share', '/usr/local/share']``
+    """
+    if ON_WINDOWS:
+        return [os_environ["ProgramData"]]
+    return [
+        os.path.join("/usr", "share"),
+        os.path.join("/usr", "local", "share"),
+    ]
+
+
+@default_value
+def xonsh_sys_config_dir(env):
+    """
+    On Linux & Mac OSX: ``'/etc/xonsh'``
+    On Windows: ``'%ALLUSERSPROFILE%\\\\xonsh'``
+    """
+    if ON_WINDOWS:
+        etc_path = os_environ["ALLUSERSPROFILE"]
+    else:
+        etc_path = "/etc"
+    return os.path.join(etc_path, "xonsh")
+
+
 def xonshconfig(env):
     """Ensures and returns the $XONSHCONFIG"""
     xcd = env.get("XONSH_CONFIG_DIR")
@@ -579,17 +614,16 @@ def xonshconfig(env):
 
 
 @default_value
-def default_xonshrc(env):
-    """Creates a new instance of the default xonshrc tuple."""
-    xcdrc = os.path.join(xonsh_config_dir(env), "rc.xsh")
-    if ON_WINDOWS:
-        dxrc = (
-            os.path.join(os_environ["ALLUSERSPROFILE"], "xonsh", "xonshrc"),
-            xcdrc,
-            os.path.expanduser("~/.xonshrc"),
-        )
-    else:
-        dxrc = ("/etc/xonshrc", xcdrc, os.path.expanduser("~/.xonshrc"))
+def default_xonshrc(env) -> "tuple[str, ...]":
+    """
+    ``['$XONSH_SYS_CONFIG_DIR/xonshrc', '$XONSH_CONFIG_DIR/xonsh/rc.xsh', '~/.xonshrc']``
+    """
+
+    dxrc = (
+        os.path.join(xonsh_sys_config_dir(env), "xonshrc"),
+        os.path.join(xonsh_config_dir(env), "rc.xsh"),
+        os.path.expanduser("~/.xonshrc"),
+    )
     # Check if old config file exists and issue warning
     old_config_filename = xonshconfig(env)
     if os.path.isfile(old_config_filename):
@@ -602,13 +636,38 @@ def default_xonshrc(env):
     return dxrc
 
 
+def get_config_paths(env: "Env", name: str):
+    return (
+        os.path.join(xonsh_sys_config_dir(env), name),
+        os.path.join(xonsh_config_dir(env), name),
+    )
+
+
 @default_value
 def default_xonshrcdir(env):
-    xdgrcd = os.path.join(xonsh_config_dir(env), "rc.d")
-    if ON_WINDOWS:
-        return (os.path.join(os_environ["ALLUSERSPROFILE"], "xonsh", "rc.d"), xdgrcd)
-    else:
-        return ("/etc/xonsh/rc.d", xdgrcd)
+    """``['$XONSH_SYS_CONFIG_DIR/rc.d', '$XONSH_CONFIG_DIR/rc.d']``\n"""
+    return get_config_paths(env, "rc.d")
+
+
+@default_value
+def default_completer_dirs(env):
+    """By default, the following paths are searched.
+    1. ``$XONSH_CONFIG_DIR/completions`` - user level completions
+    2. ``$XONSH_SYS_CONFIG_DIR/completions`` - system level completions
+    3. ``$XONSH_DATA_DIR/generated_completions`` - auto generated completers from man pages
+    4. ``$XDG_DATA_DIRS/xonsh/vendor_completions`` - completers from other programs can be placed here.
+
+    Other than this, Python package namespace ``xompletions`` can be used to put completer modules as well.
+    """
+    # inspired from - https://fishshell.com/docs/current/completions.html#where-to-put-completions
+    return [
+        os.path.join(env["XONSH_CONFIG_DIR"], "completions"),
+        os.path.join(env["XONSH_SYS_CONFIG_DIR"], "completions"),
+        os.path.join(env["XONSH_DATA_DIR"], "generated_completions"),
+    ] + [
+        os.path.join(parent, "xonsh", "vendor_completions")
+        for parent in env["XDG_DATA_DIRS"]
+    ]
 
 
 @default_value
@@ -630,6 +689,13 @@ def default_lscolors(env):
     return lsc
 
 
+@default_value
+def default_prompt_fields(env):
+    """``xonsh.prompt.PROMPT_FIELDS``"""
+    # todo: generate document for all default fields
+    return prompt.PromptFields(XSH)
+
+
 VarKeyType = tp.Union[str, tp.Pattern]
 
 
@@ -694,6 +760,9 @@ class Var(tp.NamedTuple):
             kwargs.update(
                 {"validate": validator, "convert": convertor, "detype": detyper}
             )
+
+        if doc_default == DefaultNotGiven and is_callable_default(default):
+            doc_default = inspect.getdoc(default) or DefaultNotGiven
         return Var(default=default, doc=doc, doc_default=doc_default, **kwargs)
 
     @classmethod
@@ -851,6 +920,19 @@ class GeneralSetting(Xettings):
         "should cause an end to execution. This is less useful at a terminal. "
         "The error that is raised is a ``subprocess.CalledProcessError``.",
     )
+    LAST_RETURN_CODE = Var.with_default(
+        0,
+        "Integer return code of the last command. Only updated during interactive use, i.e. not during execution of scripts.",
+    )
+
+    SHLVL = Var(
+        is_valid_shlvl,
+        to_shlvl,
+        str,
+        0,
+        "Shell nesting level typed as integer, mirrors bash's $SHLVL.",
+        is_configurable=False,
+    )
     XONSH_SUBPROC_CAPTURED_PRINT_STDERR = Var.with_default(
         False,
         "If ``True`` the stderr from captured subproc will be printed automatically.",
@@ -876,7 +958,8 @@ class GeneralSetting(Xettings):
     XONSH_CAPTURE_ALWAYS = Var.with_default(
         False,
         "Try to capture output of commands run without explicit capturing.\n"
-        "If True, xonsh will capture the output of commands run directly or in ``![]``.\n"
+        "If True, xonsh will capture the output of commands run directly or in ``![]``"
+        "to the session history.\n"
         "Setting to True has the following disadvantages:\n"
         "* Some interactive commands won't work properly (like when ``git`` invokes an interactive editor).\n"
         "  For more information see discussion at https://github.com/xonsh/xonsh/issues/3672.\n"
@@ -928,29 +1011,35 @@ class GeneralSetting(Xettings):
         doc_default="``~/.local/share``",
         type_str="str",
     )
+    XDG_DATA_DIRS = Var.with_default(
+        xdg_data_dirs,
+        "A list of directories where system level data files are stored.",
+        type_str="env_path",
+    )
     XONSHRC = Var.with_default(
         default_xonshrc,
         "A list of the locations of run control files, if they exist.  User "
         "defined run control file will supersede values set in system-wide "
         "control file if there is a naming collision. $THREAD_SUBPROCS=None "
         "when reading in run control files.",
-        doc_default=(
-            "On Linux & Mac OSX: ``['/etc/xonshrc', '~/.config/xonsh/rc.xsh', '~/.xonshrc']``\n"
-            "\nOn Windows: "
-            "``['%ALLUSERSPROFILE%\\\\xonsh\\\\xonshrc', '~/.config/xonsh/rc.xsh', '~/.xonshrc']``"
-        ),
         type_str="env_path",
     )
     XONSHRC_DIR = Var.with_default(
         default_xonshrcdir,
-        "A list of directories, from which all .xsh files will be loaded "
+        "A list of directories, from which all .xsh|.py files will be loaded "
         "at startup, sorted in lexographic order. Files in these directories "
         "are loaded after any files in XONSHRC.",
-        doc_default=(
-            "On Linux & Mac OSX: ``['/etc/xonsh/rc.d', '~/.config/xonsh/rc.d']``\n"
-            "On Windows: "
-            "``['%ALLUSERSPROFILE%\\\\xonsh\\\\rc.d', '~/.config/xonsh/rc.d']``"
-        ),
+        type_str="env_path",
+    )
+    XONSH_COMPLETER_DIRS = Var.with_default(
+        default_completer_dirs,
+        """\
+A list of paths where Xonsh searches for command completions.
+Any completions defined are lazy loaded when needed.
+The name of the completer file should match that of the completing command.
+The file should contain a function with the signature
+``xonsh_complete(ctx: CommandContext) -> Iterator[RichCompletion|str]``.
+""",
         type_str="env_path",
     )
     XONSH_APPEND_NEWLINE = Var.with_default(
@@ -971,9 +1060,14 @@ class GeneralSetting(Xettings):
     )
     XONSH_CONFIG_DIR = Var.with_default(
         xonsh_config_dir,
-        "This is the location where xonsh configuration information is stored.",
+        "This is the location where xonsh user-level configuration information is stored.",
+        is_configurable=False,
+        type_str="str",
+    )
+    XONSH_SYS_CONFIG_DIR = Var.with_default(
+        xonsh_sys_config_dir,
+        "This is the location where xonsh system-level configuration information is stored.",
         is_configurable=False,
-        doc_default="``$XDG_CONFIG_HOME/xonsh``",
         type_str="str",
     )
     XONSH_COLOR_STYLE = Var.with_default(
@@ -999,14 +1093,9 @@ class GeneralSetting(Xettings):
         "presented, like PLY parsing messages.",
         is_configurable=False,
     )
-    XONSH_NO_AMALGAMATE = Var.with_default(
-        False,
-        "Setting this variable prior to starting xonsh to a truthy value will suppress amalgamated imports.",
-        is_configurable=False,
-    )
     XONSH_DATA_DIR = Var.with_default(
         xonsh_data_dir,
-        "This is the location where xonsh data files are stored, such as " "history.",
+        "This is the location where xonsh data files are stored, such as history, generated completers ...",
         doc_default="``$XDG_DATA_HOME/xonsh``",
         type_str="str",
     )
@@ -1089,8 +1178,9 @@ class GeneralSetting(Xettings):
 By default it just prints ``cmds`` like below.
 
 .. code-block:: python
-def tracer(cmds: list, captured: Union[bool, str]):
-    print(f"TRACE SUBPROC: {cmds}, captured={captured}", file=sys.stderr)
+
+    def tracer(cmds: list, captured: Union[bool, str]):
+        print(f"TRACE SUBPROC: {cmds}, captured={captured}", file=sys.stderr)
 """,
     )
     XONSH_TRACEBACK_LOGFILE = Var(
@@ -1103,6 +1193,17 @@ def tracer(cmds: list, captured: Union[bool, str]):
         "or None / the empty string if traceback logging is not desired. "
         "Logging to a file is not enabled by default.",
     )
+    XONTRIBS_AUTOLOAD_DISABLED = Var(
+        default=False,
+        doc="""\
+Controls auto-loading behaviour of xontrib packages at the startup.
+* Set this to ``True`` to disable autoloading completely.
+* Setting this to a list of xontrib names will block loading those specifically.
+""",
+        doc_default="""\
+Xontribs with ``xonsh.xontrib`` entrypoint will be loaded automatically by default.
+""",
+    )
     STAR_PATH = Var.no_default("env_path", pattern=re.compile(r"\w*PATH$"))
     STAR_DIRS = Var.no_default("env_path", pattern=re.compile(r"\w*DIRS$"))
 
@@ -1276,16 +1377,15 @@ class PromptSetting(Xettings):
         "This value is never inherited from parent processes.",
         doc_default="``xonsh.environ.DEFAULT_PROMPT``",
     )
-    PROMPT_FIELDS = Var(
-        always_true,
-        None,
-        None,
-        prompt.PROMPT_FIELDS,
-        "Dictionary containing variables to be used when formatting $PROMPT "
+    PROMPT_FIELDS = Var.with_default(
+        default_prompt_fields,
+        validate=always_true,
+        convert=None,
+        detype=None,
+        doc="Dictionary containing variables to be used when formatting $PROMPT "
         "and $TITLE. See 'Customizing the Prompt' "
         "http://xon.sh/tutorial.html#customizing-the-prompt",
         is_configurable=False,
-        doc_default="``xonsh.prompt.PROMPT_FIELDS``",
     )
     PROMPT_REFRESH_INTERVAL = Var.with_default(
         0.0,  # keep as float
@@ -1335,7 +1435,8 @@ class PromptSetting(Xettings):
         "To use the ``prompt_toolkit`` shell you need to have the "
         "`prompt_toolkit <https://github.com/jonathanslenders/python-prompt-toolkit>`_"
         " library installed. To specify which shell should be used, do so in "
-        "the run control file.",
+        "the run control file. "
+        "It also accepts a class type that inherits from ``xonsh.base_shell.BaseShell``",
         doc_default="``best``",
     )
     SUGGEST_COMMANDS = Var.with_default(
@@ -1395,28 +1496,6 @@ class PromptSetting(Xettings):
         "Path to the currently active Python environment.",
         is_configurable=False,
     )
-    XONSH_GITSTATUS_ = Var.with_default(
-        None,
-        "Symbols for gitstatus prompt. Default values are: \n\n"
-        + "\n".join(
-            (
-                f"* ``XONSH_GITSTATUS_{fld.name}``: ``{fld.value}``"
-                for fld in GITSTATUS_FIELD_DEFS
-            )
-        ),
-        pattern="XONSH_GITSTATUS_*",
-    )
-    XONSH_GITSTATUS_FIELDS_HIDDEN = Var.with_default(
-        (),
-        "Fields to hide in {gitstatus} prompt (all fields below are shown by default.) \n\n"
-        + "\n".join(
-            (
-                f"* ``{fld.name}``\n"
-                for fld in GITSTATUS_FIELD_DEFS
-                if not fld.name.startswith("HASH")
-            )
-        ),
-    )
     XONSH_HISTORY_MATCH_ANYWHERE = Var.with_default(
         False,
         "When searching history from a partial string (by pressing up arrow), "
@@ -1468,12 +1547,13 @@ class PromptHistorySetting(Xettings):
         set(),
         "A set of strings (comma-separated list in string form) of options "
         "that determine what commands are saved to the history list. By "
-        "default all commands are saved. The option ``ignoredups`` will not "
-        "save the command if it matches the previous command. The option "
-        "``ignoreerr`` will cause any commands that fail (i.e. return non-zero "
-        "exit status) to not be added to the history list. The option "
-        "``erasedups`` will remove all previous commands that matches and updates the frequency. "
-        "Note: ``erasedups`` is supported only in sqlite backend).",
+        "default all commands are saved. Options are as follows:\n\n"
+        "- ``ignoredups`` will not save the command if it matches the previous command\n"
+        "- ``ignoreerr`` will cause any commands that fail (i.e. return non-zero "
+        "exit status) to not be added to the history list\n"
+        "- ``ignorespace`` will not save the command if it begins with a space\n"
+        "- ``erasedups`` will remove all previous commands that matches and updates the frequency "
+        "(Note: only supported in sqlite backend)",
         can_store_as_str=True,
     )
     XONSH_HISTORY_SIZE = Var(
@@ -1492,8 +1572,8 @@ class PromptHistorySetting(Xettings):
     )
     XONSH_STORE_STDOUT = Var.with_default(
         False,
-        "Whether or not to store the ``stdout`` and ``stderr`` streams in the "
-        "history files.",
+        "Store the ``stdout`` and ``stderr`` streams to the history. "
+        "Requires that XONSH_CAPTURE_ALWAYS is True.",
     )
     XONSH_HISTORY_SAVE_CWD = Var.with_default(
         True,
@@ -1595,8 +1675,23 @@ class AutoCompletionSetting(Xettings):
     """Tab-completion behavior."""
 
     ALIAS_COMPLETIONS_OPTIONS_BY_DEFAULT = Var.with_default(
-        doc="If True, Argparser based alias completions will show options (e.g. -h, ...) without "
-        "requesting explicitly with option prefix (-).",
+        doc="""\
+If True, :py:class:`xonsh.completers.argparser.ArgparseCompleter` based completions
+will show options (e.g. -h, ...) without requesting explicitly with option prefix (e.g. '-').""",
+        default=False,
+    )
+    ALIAS_COMPLETIONS_OPTIONS_LONGEST = Var.with_default(
+        doc="""\
+Whether to show all options or just the longest for
+the :py:class:`xonsh.completers.argparser.ArgparseCompleter` based completions.
+For example, with ``-h``, ``--help`` both denoting ``help`` action.
+If True, then only ``--help`` is shown.
+This is to reduce the noise in generated completions.""",
+        default=False,
+    )
+    CMD_COMPLETIONS_SHOW_DESC = Var.with_default(
+        doc="If True, command completions will show description part with path to the binary and alias in case of "
+        "xonsh functions.",
         default=False,
         type_str="bool",
     )
@@ -1609,7 +1704,7 @@ class AutoCompletionSetting(Xettings):
         "For both bash-completion v1.x and v2.x, paths of individual completion "
         "scripts (like ``.../completes/ssh``) do not need to be included here. "
         "The default values are platform "
-        "dependent, but sane. To specify an alternate list, do so in the run "
+        "dependent, but reasonable. To specify an alternate list, do so in the run "
         "control file.",
         default=BASH_COMPLETIONS_DEFAULT,
         doc_default=(
@@ -1617,7 +1712,8 @@ class AutoCompletionSetting(Xettings):
             "    ``('/usr/share/bash-completion/bash_completion', )``\n\n"
             "But, on Mac it is:\n\n"
             "    ``('/usr/local/share/bash-completion/bash_completion', "
-            "'/usr/local/etc/bash_completion')``\n\n"
+            "'/usr/local/etc/bash_completion', "
+            "'/opt/homebrew/share/bash-completion/bash_completion'),``\n\n"
             "Other OS-specific defaults may be added in the future."
         ),
         type_str="env_path",
@@ -1788,7 +1884,7 @@ class Env(cabc.MutableMapping):
             self[key] = val
         if ON_WINDOWS:
             path_key = next((k for k in self._d if k.upper() == "PATH"), None)
-            if path_key:
+            if path_key is not None:
                 self["PATH"] = self._d.pop(path_key)
         if "PATH" not in self._d:
             # this is here so the PATH is accessible to subprocs and so that
@@ -2068,17 +2164,22 @@ class Env(cabc.MutableMapping):
         else:
             return default
 
+    def get_stringified(self, key, default=None):
+        value = self.get(key, default)
+        detyper = self.get_detyper(key)
+        return detyper(value)
+
     def rawkeys(self):
         """An iterator that returns all environment keys in their original form.
         This include string & compiled regular expression keys.
         """
         yield from (
             set(self._d)
-            | set(
+            | {
                 k
                 for k in self._vars.keys()
                 if self._vars[k].default is not DefaultNotGiven
-            )
+            }
         )
 
     def __iter__(self):
@@ -2098,7 +2199,7 @@ class Env(cabc.MutableMapping):
         return str(self._d)
 
     def __repr__(self):
-        return "{0}.{1}(...)".format(self.__class__.__module__, self.__class__.__name__)
+        return f"{self.__class__.__module__}.{self.__class__.__name__}(...)"
 
     def __hash__(self) -> int:
         return hash(str(self._d))
@@ -2193,6 +2294,11 @@ class Env(cabc.MutableMapping):
         """
         self._vars.pop(name)
 
+    def is_configurable(self, name):
+        if name not in self._vars:
+            return False
+        return self._vars[name].is_configurable
+
 
 class InternalEnvironDict(ChainMap):
     """A dictionary which supports thread-local overrides.
@@ -2288,6 +2394,15 @@ def locate_binary(name):
     return XSH.commands_cache.locate_binary(name)
 
 
+def scan_dir_for_source_files(path: str):
+    if not os.path.isdir(path):
+        return
+    with os.scandir(path) as it:
+        for entry in it:
+            if entry.is_file() and (entry.name.endswith((".py", ".xsh"))):
+                yield os.path.join(path, entry.name), entry
+
+
 def xonshrc_context(
     rcfiles=None, rcdirs=None, execer=None, ctx=None, env=None, login=True
 ):
@@ -2308,15 +2423,15 @@ def xonshrc_context(
                 )
                 if status:
                     loaded.append(rcfile)
+
     if rcdirs is not None:
         for rcdir in rcdirs:
-            if os.path.isdir(rcdir):
-                for rcfile in sorted(glob.glob(os.path.join(rcdir, "*.xsh"))):
-                    status = xonsh_script_run_control(
-                        rcfile, ctx, env, execer=execer, login=login
-                    )
-                    if status:
-                        loaded.append(rcfile)
+            for rcfile in sorted(dict(scan_dir_for_source_files(rcdir))):
+                status = xonsh_script_run_control(
+                    rcfile, ctx, env, execer=execer, login=login
+                )
+                if status:
+                    loaded.append(rcfile)
     if env["THREAD_SUBPROCS"] is None:
         env["THREAD_SUBPROCS"] = orig_thread
     return loaded
@@ -2354,26 +2469,30 @@ class _RcPath(str):
 
 
 def xonsh_script_run_control(filename, ctx, env, execer=None, login=True):
-    """Loads a xonsh file and applies it as a run control."""
+    """Loads a xonsh file and applies it as a run control.
+    Any exceptions are logged here, returns boolean indicating success.
+    """
     if execer is None:
         return False
     updates = {"__file__": filename, "__name__": os.path.abspath(filename)}
     rc_dir = _RcPath(os.path.dirname(filename))
     sys.path.append(rc_dir)
-    try:
-        with swap_values(ctx, updates):
-            run_script_with_cache(filename, execer, ctx)
-        loaded = True
-    except SyntaxError as err:
-        msg = "syntax error in xonsh run control file {0!r}: {1!s}"
-        print_exception(msg.format(filename, err))
-        loaded = False
-    except Exception as err:
-        msg = "error running xonsh run control file {0!r}: {1!s}"
-        print_exception(msg.format(filename, err))
-        loaded = False
-    finally:
-        sys.path = list(filter(lambda p: p is not rc_dir, sys.path))
+    with swap_values(ctx, updates):
+        try:
+            exc_info = run_script_with_cache(filename, execer, ctx)
+        except SyntaxError:
+            exc_info = sys.exc_info()
+        if exc_info != (None, None, None):
+            err_type, err, _ = exc_info
+            loaded = False
+            if err_type is SyntaxError:
+                msg = "syntax error in xonsh run control file {0!r}: {1!s}"
+            else:
+                msg = "error running xonsh run control file {0!r}: {1!s}"
+            print_exception(msg.format(filename, err), exc_info=exc_info)
+        else:
+            loaded = True
+    sys.path = list(filter(lambda p: p is not rc_dir, sys.path))
     return loaded
 
 
@@ -2382,7 +2501,7 @@ def default_env(env=None):
     # in order of increasing precedence
     ctx = {
         "BASH_COMPLETIONS": list(DEFAULT_VARS["BASH_COMPLETIONS"].default),
-        "PROMPT_FIELDS": dict(DEFAULT_VARS["PROMPT_FIELDS"].default),
+        "PROMPT_FIELDS": DEFAULT_VARS["PROMPT_FIELDS"].default(env),
         "XONSH_VERSION": XONSH_VERSION,
     }
     ctx.update(os_environ)
@@ -2395,6 +2514,9 @@ def default_env(env=None):
         del ctx["PROMPT"]
     except KeyError:
         pass
+    # increment $SHLVL
+    old_shlvl = to_shlvl(ctx.get("SHLVL", None))
+    ctx["SHLVL"] = adjust_shlvl(old_shlvl, 1)
     # finalize env
     if env is not None:
         ctx.update(env)
