@@ -1,22 +1,22 @@
-# -*- coding: utf-8 -*-
 """Job control for the xonsh shell."""
+import collections
+import ctypes
 import os
+import signal
+import subprocess
 import sys
 import time
-import ctypes
-import signal
-import builtins
-import argparse
-import subprocess
-import collections
 import typing as tp
 
+from xonsh.built_ins import XSH
+from xonsh.cli_utils import Annotated, Arg, ArgParserAlias
+from xonsh.completers.tools import RichCompletion
 from xonsh.lazyasd import LazyObject
-from xonsh.platform import FD_STDERR, ON_DARWIN, ON_WINDOWS, ON_CYGWIN, ON_MSYS, LIBC
+from xonsh.platform import FD_STDERR, LIBC, ON_CYGWIN, ON_DARWIN, ON_MSYS, ON_WINDOWS
 from xonsh.tools import unthreadable
 
-
-tasks = LazyObject(collections.deque, globals(), "tasks")
+# there is not much cost initing deque
+tasks: tp.Deque[int] = collections.deque()
 # Track time stamp of last exit command, so that two consecutive attempts to
 # exit can kill all jobs and exit.
 _last_exit_time: tp.Optional[float] = None
@@ -36,7 +36,6 @@ if ON_DARWIN:
             except ProcessLookupError:
                 pass
 
-
 elif ON_WINDOWS:
     pass
 elif ON_CYGWIN or ON_MSYS:
@@ -52,7 +51,6 @@ elif ON_CYGWIN or ON_MSYS:
                 except Exception:
                     pass
 
-
 else:
 
     def _send_signal(job, signal):
@@ -78,6 +76,8 @@ if ON_WINDOWS:
             stderr=subprocess.STDOUT,
         )
 
+    _hup = _kill  # there is no equivalent of SIGHUP on Windows
+
     def ignore_sigtstp():
         pass
 
@@ -107,7 +107,6 @@ if ON_WINDOWS:
                     pass  # ignore error if process closed before we got here
         return wait_for_active_job(last_task=active_task)
 
-
 else:
 
     def _continue(job):
@@ -117,6 +116,9 @@ else:
     def _kill(job):
         _send_signal(job, signal.SIGKILL)
 
+    def _hup(job):
+        _send_signal(job, signal.SIGHUP)
+
     def ignore_sigtstp():
         signal.signal(signal.SIGTSTP, signal.SIG_IGN)
 
@@ -243,7 +245,7 @@ def _safe_wait_for_active_job(last_task=None, backgrounded=False):
 
 
 def get_next_task():
-    """ Get the next active task and put it on top of the queue"""
+    """Get the next active task and put it on top of the queue"""
     _clear_dead_jobs()
     selected_task = None
     for tid in tasks:
@@ -259,7 +261,7 @@ def get_next_task():
 
 
 def get_task(tid):
-    return builtins.__xonsh__.all_jobs[tid]
+    return XSH.all_jobs[tid]
 
 
 def _clear_dead_jobs():
@@ -270,29 +272,34 @@ def _clear_dead_jobs():
             to_remove.add(tid)
     for job in to_remove:
         tasks.remove(job)
-        del builtins.__xonsh__.all_jobs[job]
+        del XSH.all_jobs[job]
+
+
+def format_job_string(num: int) -> str:
+    try:
+        job = XSH.all_jobs[num]
+    except KeyError:
+        return ""
+    pos = "+" if tasks[0] == num else "-" if tasks[1] == num else " "
+    status = job["status"]
+    cmd = " ".join([" ".join(i) if isinstance(i, list) else i for i in job["cmds"]])
+    pid = job["pids"][-1]
+    bg = " &" if job["bg"] else ""
+    return f"[{num}]{pos} {status}: {cmd}{bg} ({pid})"
 
 
 def print_one_job(num, outfile=sys.stdout):
     """Print a line describing job number ``num``."""
-    try:
-        job = builtins.__xonsh__.all_jobs[num]
-    except KeyError:
-        return
-    pos = "+" if tasks[0] == num else "-" if tasks[1] == num else " "
-    status = job["status"]
-    cmd = [" ".join(i) if isinstance(i, list) else i for i in job["cmds"]]
-    cmd = " ".join(cmd)
-    pid = job["pids"][-1]
-    bg = " &" if job["bg"] else ""
-    print("[{}]{} {}: {}{} ({})".format(num, pos, status, cmd, bg, pid), file=outfile)
+    info = format_job_string(num)
+    if info:
+        print(info, file=outfile)
 
 
 def get_next_job_number():
     """Get the lowest available unique job number (for the next job created)."""
     _clear_dead_jobs()
     i = 1
-    while i in builtins.__xonsh__.all_jobs:
+    while i in XSH.all_jobs:
         i += 1
     return i
 
@@ -303,26 +310,26 @@ def add_job(info):
     info["started"] = time.time()
     info["status"] = "running"
     tasks.appendleft(num)
-    builtins.__xonsh__.all_jobs[num] = info
-    if info["bg"] and builtins.__xonsh__.env.get("XONSH_INTERACTIVE"):
+    XSH.all_jobs[num] = info
+    if info["bg"] and XSH.env.get("XONSH_INTERACTIVE"):
         print_one_job(num)
 
 
 def clean_jobs():
     """Clean up jobs for exiting shell
 
-    In non-interactive mode, kill all jobs.
+    In non-interactive mode, send SIGHUP to all jobs.
 
     In interactive mode, check for suspended or background jobs, print a
     warning if any exist, and return False. Otherwise, return True.
     """
     jobs_clean = True
-    if builtins.__xonsh__.env["XONSH_INTERACTIVE"]:
+    if XSH.env["XONSH_INTERACTIVE"]:
         _clear_dead_jobs()
 
-        if builtins.__xonsh__.all_jobs:
+        if XSH.all_jobs:
             global _last_exit_time
-            hist = builtins.__xonsh__.history
+            hist = XSH.history
             if hist is not None and len(hist.tss) > 0:
                 last_cmd_start = hist.tss[-1][0]
             else:
@@ -333,18 +340,18 @@ def clean_jobs():
                 # part of the last command and is now being called again
                 # immediately. Kill jobs and exit without reminder about
                 # unfinished jobs in this case.
-                kill_all_jobs()
+                hup_all_jobs()
             else:
-                if len(builtins.__xonsh__.all_jobs) > 1:
+                if len(XSH.all_jobs) > 1:
                     msg = "there are unfinished jobs"
                 else:
                     msg = "there is an unfinished job"
 
-                if builtins.__xonsh__.env["SHELL_TYPE"] != "prompt_toolkit":
+                if XSH.env["SHELL_TYPE"] != "prompt_toolkit":
                     # The Ctrl+D binding for prompt_toolkit already inserts a
                     # newline
                     print()
-                print("xonsh: {}".format(msg), file=sys.stderr)
+                print(f"xonsh: {msg}", file=sys.stderr)
                 print("-" * 5, file=sys.stderr)
                 jobs([], stdout=sys.stderr)
                 print("-" * 5, file=sys.stderr)
@@ -355,18 +362,18 @@ def clean_jobs():
                 jobs_clean = False
                 _last_exit_time = time.time()
     else:
-        kill_all_jobs()
+        hup_all_jobs()
 
     return jobs_clean
 
 
-def kill_all_jobs():
+def hup_all_jobs():
     """
-    Send SIGKILL to all child processes (called when exiting xonsh).
+    Send SIGHUP to all child processes (called when exiting xonsh).
     """
     _clear_dead_jobs()
-    for job in builtins.__xonsh__.all_jobs.values():
-        _kill(job)
+    for job in XSH.all_jobs.values():
+        _hup(job)
 
 
 def jobs(args, stdin=None, stdout=sys.stdout, stderr=None):
@@ -381,7 +388,7 @@ def jobs(args, stdin=None, stdout=sys.stdout, stderr=None):
     return None, None
 
 
-def resume_job(args, wording):
+def resume_job(args, wording: tp.Literal["fg", "bg"]):
     """
     used by fg and bg to resume a job either in the foreground or in the background.
     """
@@ -400,12 +407,12 @@ def resume_job(args, wording):
             else:
                 tid = int(args[0])
         except (ValueError, IndexError):
-            return "", "Invalid job: {}\n".format(args[0])
+            return "", f"Invalid job: {args[0]}\n"
 
-        if tid not in builtins.__xonsh__.all_jobs:
-            return "", "Invalid job: {}\n".format(args[0])
+        if tid not in XSH.all_jobs:
+            return "", f"Invalid job: {args[0]}\n"
     else:
-        return "", "{} expects 0 or 1 arguments, not {}\n".format(wording, len(args))
+        return "", f"{wording} expects 0 or 1 arguments, not {len(args)}\n"
 
     # Put this one on top of the queue
     tasks.remove(tid)
@@ -414,10 +421,12 @@ def resume_job(args, wording):
     job = get_task(tid)
     job["bg"] = False
     job["status"] = "running"
-    if builtins.__xonsh__.env.get("XONSH_INTERACTIVE"):
+    if XSH.env.get("XONSH_INTERACTIVE"):
         print_one_job(tid)
     pipeline = job["pipeline"]
-    pipeline.resume(job)
+    pipeline.resume(
+        job, tee_output=(wording == "fg")
+    )  # do not tee output for background jobs
 
 
 @unthreadable
@@ -447,53 +456,47 @@ def bg(args, stdin=None):
         return res
 
 
-def disown(args, stdin=None):
-    """
-    xonsh command: disown
+def job_id_completer(xsh, **_):
+    """Return currently running jobs ids"""
+    for job_id in xsh.all_jobs:
+        yield RichCompletion(str(job_id), description=format_job_string(job_id))
 
-    Remove the specified jobs from the job table; the shell will no longer
+
+def disown_fn(
+    job_ids: Annotated[
+        tp.Sequence[int], Arg(type=int, nargs="*", completer=job_id_completer)
+    ],
+    force_auto_continue=False,
+):
+    """Remove the specified jobs from the job table; the shell will no longer
     report their status, and will not complain if you try to exit an
-    interactive shell with them running or stopped. If no job is specified,
-    disown the current job.
+    interactive shell with them running or stopped.
 
     If the jobs are currently stopped and the $AUTO_CONTINUE option is not set
     ($AUTO_CONTINUE = False), a warning is printed containing information about
     how to make them continue after they have been disowned.
 
-    Specifying the -c or --continue option for this command is equivalent to
-    setting $AUTO_CONTINUE=True.
+    Parameters
+    ----------
+    job_ids
+        Jobs to act on or none to disown the current job
+    force_auto_continue : -c, --continue
+        Automatically continue stopped jobs when they are disowned, equivalent to setting $AUTO_CONTINUE=True
     """
 
-    parser = argparse.ArgumentParser("disown", description=disown.__doc__)
-    parser.add_argument(
-        "job_ids",
-        type=int,
-        nargs="*",
-        help="Jobs to act on or none to use the current job",
-    )
-    parser.add_argument(
-        "-c",
-        "--continue",
-        action="store_true",
-        dest="force_auto_continue",
-        help="Automatically continue stopped jobs when they are disowned",
-    )
-
-    pargs = parser.parse_args(args)
-
     if len(tasks) == 0:
         return "", "There are no active jobs"
 
     messages = []
     # if args.job_ids is empty, use the active task
-    for tid in pargs.job_ids or [tasks[0]]:
+    for tid in job_ids or [tasks[0]]:
         try:
             current_task = get_task(tid)
         except KeyError:
             return "", f"'{tid}' is not a valid job ID"
 
-        auto_cont = builtins.__xonsh__.env.get("AUTO_CONTINUE", False)
-        if auto_cont or pargs.force_auto_continue:
+        auto_cont = XSH.env.get("AUTO_CONTINUE", False)
+        if auto_cont or force_auto_continue:
             _continue(current_task)
         elif current_task["status"] == "stopped":
             messages.append(
@@ -504,8 +507,11 @@ def disown(args, stdin=None):
 
         # Stop tracking this task
         tasks.remove(tid)
-        del builtins.__xonsh__.all_jobs[tid]
+        del XSH.all_jobs[tid]
         messages.append(f"Removed job {tid} ({current_task['status']})")
 
     if messages:
         return "".join(messages)
+
+
+disown = ArgParserAlias(prog="disown", func=disown_fn, has_args=True)
